'' =================================================================================================
''
''   File....... jm_165_595_pwm.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2013 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 20 NOV 2013
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq               ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us


con { io pins }

  RX1     = 31                                                  ' programming/debug
  TX1     = 30
  SDA     = 29                                                  ' boot eeprom
  SCL     = 28

  O_DAT   = 27                                                  ' x595 outputs
  O_CLK   = 26
  O_LATCH = 25

  I_DAT   =  9                                                  ' x165 inputs
  I_LOAD  =  8
  I_CLK   =  7


con

  #0, LSBFIRST, MSBFIRST
    

obj

  btns : "jm_165_ez"                                            ' read 74x165s
  leds : "jm_pwm595x2"                                          ' pwm 2 74x595s
 

var


pub main | bits, idx, level

  btns.start(I_DAT, I_CLK, I_LOAD) 
  leds.start(O_DAT, O_CLK, O_LATCH, -1)
  
  repeat
    bits := btns.read(btns#MSBFIRST, 2)                         ' scan 2 165s
    idx := bit_pos(bits, LSBFIRST)                              ' find first active
    if (idx => 0)                                               ' new input?
      level := leds.read(idx)                                   ' get channel level
      if (level == 0)                                           ' if off
        fade_in(idx)                                            '  fade in
      else                                                      ' else
        fade_out(idx)                                           '  fade out
    

pub bit_pos(value, mode)

'' Returns position of 1st "1" bit
'' -- mode 0 (LSBFIRST) to scan from lsb, mode 1 (MSBFIRST) to scan from msb 
'' -- -1 = no bits set

  if (value == 0)                                               ' if no bits                                   
    return -1                                                   '  return -1

  else
    if (mode == LSBFIRST)                                       ' check from LSB
      value ><= 32                                              '  flip for >|
      return (32 - >|value)
    else
      return (>|value - 1)
  

pub fade_in(ch) | level

  repeat level from 0 to 255
    leds.set(ch, leds.ez_log(level))                    
    pause(4)


pub fade_out(ch) | level  

  repeat level from 255 to 0
    leds.set(ch, leds.ez_log(level))                    
    pause(4)


con

  { ------------- }
  {  B A S I C S  }
  { ------------- }


pub pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                   ' delay must be > 0
    return
  else
    t := cnt - 1776                                             ' sync with system counter
    repeat ms                                                   ' run delay
      waitcnt(t += MS_001)
    

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]
  

dat { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}